@@ -1,5 +1,5 @@
 ;;
-;; Copyright (C) 2011-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+;; Copyright (C) 2011-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;;
 ;; This program is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by the
@@ -19,7 +19,9 @@
 (define-module (mu plot)
   :use-module (mu)
   :use-module (ice-9 popen)
-  :export (mu:plot))
+  :export ( mu:plot ;; alias for mu:plot-histogram
+	    mu:plot-histogram
+	    ))
 
 (define (export-pairs pairs)
   "Write a temporary file with the list of PAIRS in table format, and
@@ -33,18 +35,37 @@ return the file name."
     (close output)
     datafile))
 
-(define* (mu:plot data title x-label y-label #:optional (ascii #f))
-  "Plot DATA with TITLE, X-LABEL and X-LABEL. If ASCII is true, display
-using raw text, otherwise, use a graphical window."
+(define (find-program-in-path prog)
+  "Find exutable program PROG in PATH; return the full path, or #f if
+not found."
+  (let* ((path (parse-path (getenv "PATH")))
+	  (progpath (search-path path prog)))
+    (if (not progpath)
+      #f
+      (if (access? progpath X_OK) ;; is
+	progpath
+	#f))))
+2
+(define* (mu:plot-histogram data title x-label y-label #:optional (text-only #f) (extra-gnuplot-opts '()))
+  "Plot DATA with TITLE, X-LABEL and X-LABEL. If TEXT-ONLY is true,
+display using raw text, otherwise, use a graphical window. DATA is a
+list of cons-pairs (X . Y)."
+  (if (not (find-program-in-path "gnuplot"))
+    (error "cannot find 'gnuplot' in path"))
+ 
   (let ((datafile (export-pairs data))
 	 (gnuplot (open-pipe "gnuplot -p" OPEN_WRITE)))
     (display (string-append
 	       "reset\n"
-	       "set term " (if ascii "dumb" "wxt") "\n"
+	       "set term " (if text-only "dumb" "wxt") "\n"
 	       "set title \"" title "\"\n"
 	       "set xlabel \"" x-label "\"\n"
 	       "set ylabel \"" y-label "\"\n"
 	       "set boxwidth 0.9\n"
+           (string-join extra-gnuplot-opts "\n")
 	       "plot \"" datafile "\" using 2:xticlabels(1) with boxes fs solid\n")
       gnuplot)
-  (close-pipe gnuplot)))
+    (close-pipe gnuplot)))
+
+;; backward compatibility
+(define mu-plot mu:plot-histogram)
