@@ -1,7 +1,7 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 
 /*
-** Copyright (C) 2008-2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,17 +52,12 @@ enum _FieldFlags {
 					    * for Xapian queries;
 					    * wildcards do NOT WORK
 					    * for such fields */
-	FLAG_XAPIAN_PREFIX_ONLY  = 1 << 7, /* whether this fields
-					    * matches only when the
-					    * prefix is explicitly
-					    * included in the search
-					    * query -- e.g., the text
-					    * body */
-	FLAG_NORMALIZE	         = 1 << 8, /* field needs flattening for
+
+	FLAG_NORMALIZE	         = 1 << 7, /* field needs flattening for
 					    * case/accents */
-	FLAG_DONT_CACHE          = 1 << 9,  /* don't cache this field in
+	FLAG_DONT_CACHE          = 1 << 8,  /* don't cache this field in
 					    * the MuMsg cache */
-	FLAG_RANGE_FIELD         = 1 << 10  /* whether this is a range field */
+	FLAG_RANGE_FIELD         = 1 << 9  /* whether this is a range field */
 
 };
 typedef enum _FieldFlags	FieldFlags;
@@ -108,7 +103,7 @@ static const MuMsgField FIELD_DATA[] = {
 	{
 		MU_MSG_FIELD_ID_BODY_HTML,
 		MU_MSG_FIELD_TYPE_STRING,
-		"bodyhtml", 'h', 0,
+		"bodyhtml", 0, 0,
 		FLAG_GMIME | FLAG_DONT_CACHE
 	},
 
@@ -124,7 +119,7 @@ static const MuMsgField FIELD_DATA[] = {
 		MU_MSG_FIELD_TYPE_TIME_T,
 		"date", 'd', 'D',
 		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_BOOLEAN | FLAG_XAPIAN_PREFIX_ONLY | FLAG_RANGE_FIELD
+		FLAG_XAPIAN_BOOLEAN  | FLAG_RANGE_FIELD
 	},
 
 	{
@@ -140,7 +135,7 @@ static const MuMsgField FIELD_DATA[] = {
 		MU_MSG_FIELD_TYPE_STRING,
 		"file" , 'j', 'J',
 		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_ESCAPE |
-		FLAG_DONT_CACHE | FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_DONT_CACHE
 	},
 
 
@@ -148,8 +143,7 @@ static const MuMsgField FIELD_DATA[] = {
 		MU_MSG_FIELD_ID_FLAGS,
 		MU_MSG_FIELD_TYPE_INT,
 		"flag", 'g', 'G',  /* flaGs */
-		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE
 	},
 
 	{
@@ -159,21 +153,22 @@ static const MuMsgField FIELD_DATA[] = {
 		FLAG_GMIME | FLAG_XAPIAN_CONTACT | FLAG_XAPIAN_VALUE
 	},
 
-	{
-		MU_MSG_FIELD_ID_PATH,
-		MU_MSG_FIELD_TYPE_STRING,
-		"path", 'l', 'L',   /* 'l' for location */
-		FLAG_GMIME | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_BOOLEAN  | FLAG_XAPIAN_PREFIX_ONLY |
-		FLAG_XAPIAN_ESCAPE
-	},
 
 	{
 		MU_MSG_FIELD_ID_MAILDIR,
 		MU_MSG_FIELD_TYPE_STRING,
 		"maildir", 'm', 'M',
 		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_ESCAPE | FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_XAPIAN_ESCAPE
+	},
+
+
+	{
+		MU_MSG_FIELD_ID_MAILING_LIST,
+		MU_MSG_FIELD_TYPE_STRING,
+		"list", 'v', 'V',
+		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
+		FLAG_XAPIAN_ESCAPE
 	},
 
 
@@ -181,23 +176,50 @@ static const MuMsgField FIELD_DATA[] = {
 		MU_MSG_FIELD_ID_MIME,
 		MU_MSG_FIELD_TYPE_STRING,
 		"mime" , 'y', 'Y',
-		FLAG_XAPIAN_TERM | FLAG_XAPIAN_ESCAPE | FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_XAPIAN_TERM | FLAG_XAPIAN_ESCAPE
 	},
 
+
+	{
+		MU_MSG_FIELD_ID_MSGID,
+		MU_MSG_FIELD_TYPE_STRING,
+		"msgid", 'i', 'I',  /* 'i' for Id */
+		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
+		FLAG_XAPIAN_ESCAPE
+	},
+
+
+	{
+		MU_MSG_FIELD_ID_PATH,
+		MU_MSG_FIELD_TYPE_STRING,
+		"path", 'l', 'L',   /* 'l' for location */
+		FLAG_GMIME | FLAG_XAPIAN_VALUE |
+		FLAG_XAPIAN_BOOLEAN | FLAG_XAPIAN_ESCAPE
+	},
+
+
 	{
 		MU_MSG_FIELD_ID_PRIO,
 		MU_MSG_FIELD_TYPE_INT,
 		"prio", 'p', 'P',
-		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE
 	},
 
+
+	{
+		MU_MSG_FIELD_ID_REFS,
+		MU_MSG_FIELD_TYPE_STRING_LIST,
+		"refs", 'r', 'R',
+		FLAG_GMIME | FLAG_XAPIAN_VALUE
+	},
+
+
 	{
 		MU_MSG_FIELD_ID_SIZE,
 		MU_MSG_FIELD_TYPE_BYTESIZE,
 		"size", 'z', 'Z', /* siZe */
 		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_PREFIX_ONLY | FLAG_RANGE_FIELD
+		FLAG_RANGE_FIELD
 	},
 
 	{
@@ -208,6 +230,22 @@ static const MuMsgField FIELD_DATA[] = {
 		FLAG_XAPIAN_TERM  | FLAG_XAPIAN_ESCAPE
 	},
 
+	{
+		MU_MSG_FIELD_ID_TAGS,
+		MU_MSG_FIELD_TYPE_STRING_LIST,
+		"tag", 'x', 'X',
+		FLAG_GMIME | FLAG_XAPIAN_TERM |	FLAG_XAPIAN_ESCAPE |
+		FLAG_XAPIAN_VALUE
+	},
+
+
+	{	/* remember which thread this message is in */
+		MU_MSG_FIELD_ID_THREAD_ID,
+		MU_MSG_FIELD_TYPE_STRING,
+		"thread", 0, 'W',
+		FLAG_XAPIAN_TERM
+	},
+
 	{
 		MU_MSG_FIELD_ID_TO,
 		MU_MSG_FIELD_TYPE_STRING,
@@ -215,34 +253,11 @@ static const MuMsgField FIELD_DATA[] = {
 		FLAG_GMIME | FLAG_XAPIAN_CONTACT | FLAG_XAPIAN_VALUE
 	},
 
-	{
-		MU_MSG_FIELD_ID_MSGID,
-		MU_MSG_FIELD_TYPE_STRING,
-		"msgid", 'i', 'I',  /* 'i' for Id */
-		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_VALUE |
-		FLAG_XAPIAN_ESCAPE | FLAG_XAPIAN_PREFIX_ONLY
-	},
-
-	{
-		MU_MSG_FIELD_ID_REFS,
-		MU_MSG_FIELD_TYPE_STRING_LIST,
-		"refs", 'r', 'R',
-		FLAG_GMIME | FLAG_XAPIAN_VALUE | FLAG_XAPIAN_PREFIX_ONLY
-	},
-
-	{
-		MU_MSG_FIELD_ID_TAGS,
-		MU_MSG_FIELD_TYPE_STRING_LIST,
-		"tag", 'x', 'X',
-		FLAG_GMIME | FLAG_XAPIAN_TERM | FLAG_XAPIAN_PREFIX_ONLY |
-		FLAG_XAPIAN_ESCAPE
-	},
-
 	{	/* special, internal field, to get a unique key */
 		MU_MSG_FIELD_ID_UID,
 		MU_MSG_FIELD_TYPE_STRING,
 		"uid", 0, 'U',
-		FLAG_XAPIAN_TERM | FLAG_XAPIAN_PREFIX_ONLY
+		FLAG_XAPIAN_TERM
 	}
 
 	/* note, mu-store also use the 'Q' internal prefix for its uids */
@@ -357,14 +372,6 @@ mu_msg_field_uses_boolean_prefix (MuMsgFieldId id)
 }
 
 
-gboolean
-mu_msg_field_needs_prefix (MuMsgFieldId id)
-{
-	g_return_val_if_fail (mu_msg_field_id_is_valid(id),FALSE);
-	return mu_msg_field(id)->_flags &
-		FLAG_XAPIAN_PREFIX_ONLY ? TRUE: FALSE;
-}
-
 
 gboolean
 mu_msg_field_is_cacheable (MuMsgFieldId id)
