@@ -1,6 +1,6 @@
 /* -*- mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 **
-** Copyright (C) 2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2012-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -198,6 +198,56 @@ get_recipient (MuMsgFile *self, GMimeRecipientType rtype)
 	return recip;
 }
 
+/*
+ * let's try to guess the mailing list from some other
+ * headers in the mail
+ */
+static gchar*
+get_fake_mailing_list (MuMsgFile *self)
+{
+	const char* hdr;
+
+	hdr = g_mime_object_get_header (GMIME_OBJECT(self->_mime_msg),
+					"X-Feed2Imap-Version");
+	if (!hdr)
+		return NULL;
+
+	/* looks like a feed2imap header; guess the source-blog
+	 * from the msgid */
+	{
+		const char *msgid, *e;
+		msgid = g_mime_message_get_message_id (self->_mime_msg);
+		if (msgid && (e = strchr (msgid, '-')))
+			return g_strndup (msgid, e - msgid);
+	}
+
+	return NULL;
+}
+
+
+static gchar*
+get_mailing_list (MuMsgFile *self)
+{
+	const char *hdr, *b, *e;
+
+	hdr = g_mime_object_get_header (GMIME_OBJECT(self->_mime_msg),
+					"List-Id");
+	if (mu_str_is_empty (hdr))
+		return get_fake_mailing_list (self);
+
+	e = NULL;
+	b = strchr (hdr, '<');
+	if (b)
+		e = strchr (b, '>');
+
+	if (b && e)
+		return g_strndup (b + 1, e - b - 1);
+	else
+		return g_strdup (hdr);
+}
+
+
+
 
 static gboolean
 looks_like_attachment (GMimeObject *part)
@@ -463,6 +513,9 @@ contains (GSList *lst, const char *str)
 }
 
 
+/*
+ * NOTE: this will get the list of references with the oldest parent
+ * at the beginning */
 static GSList*
 get_references  (MuMsgFile *self)
 {
@@ -497,9 +550,14 @@ get_references  (MuMsgFile *self)
 		g_mime_references_free (mime_refs);
 	}
 
+	/* reverse, because we used g_slist_prepend for performance
+	 * reasons */
 	return g_slist_reverse (msgids);
 }
 
+
+
+
 /* see: http://does-not-exist.org/mail-archives/mutt-dev/msg08249.html */
 static GSList*
 get_tags (MuMsgFile *self)
@@ -592,6 +650,10 @@ mu_msg_file_get_str_field (MuMsgFile *self, MuMsgFieldId mfid,
 
 	case MU_MSG_FIELD_ID_PATH: return self->_path;
 
+	case MU_MSG_FIELD_ID_MAILING_LIST:
+		*do_free = TRUE;
+		return (char*)get_mailing_list (self);
+
 	case MU_MSG_FIELD_ID_SUBJECT:
 		return (char*)maybe_cleanup
 			(g_mime_message_get_subject (self->_mime_msg),
