@@ -1,6 +1,6 @@
 /* -*- mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 **
-** Copyright (C) 2008-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -257,7 +257,7 @@ get_str_list_field (MuMsg *self, MuMsgFieldId mfid)
 
 	if (self->_doc && mu_msg_field_xapian_value (mfid))
 		val = mu_msg_doc_get_str_list_field (self->_doc, mfid);
-	if (!val) {
+	else if (mu_msg_field_gmime (mfid)) {
 		/* if we don't have a file object yet, we need to
 		 * create it from the file on disk */
 		if (!mu_msg_load_msg_file (self, NULL))
@@ -287,10 +287,8 @@ get_str_field (MuMsg *self, MuMsgFieldId mfid)
 		if (!mu_msg_load_msg_file (self, NULL))
 			return NULL;
 		val = mu_msg_file_get_str_field (self->_file, mfid, &do_free);
-	} else {
-		g_warning ("%s: cannot retrieve field", __FUNCTION__);
+	} else
 		val = NULL;
-	}
 
 	return do_free ? free_later_str (self, val) : val;
 }
@@ -373,6 +371,16 @@ mu_msg_get_msgid  (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_MSGID);
 }
 
+
+
+const char*
+mu_msg_get_mailing_list (MuMsg *self)
+{
+	g_return_val_if_fail (self, NULL);
+	return get_str_field (self, MU_MSG_FIELD_ID_MAILING_LIST);
+}
+
+
 const char*
 mu_msg_get_maildir (MuMsg *self)
 {
