@@ -44,8 +44,12 @@
 
     (define-key map "m" 'mu4e~main-toggle-mail-sending-mode)
     (define-key map "f" 'smtpmail-send-queued-mail)
-    (define-key map "U" 'mu4e-update-mail-show-window)
 
+    ;;
+    (define-key map "U" 'mu4e-update-mail-and-index)
+    (define-key map  (kbd "C-S-u") 'mu4e-update-mail-and-index)
+    
+    
     (define-key map "$" 'mu4e-show-log)
     (define-key map "A" 'mu4e-about)
     (define-key map "H" 'mu4e-display-manual)
@@ -65,10 +69,10 @@
 
 
 (defun mu4e~main-action-str (str &optional func-or-shortcut)
-  "Highlight the first occurence of [..] in STR. If
-FUNC-OR-SHORTCUT is non-nil and if it is a function, call it when
-STR is clicked (using RET or mouse-2); if FUNC-OR-SHORTCUT is a
-string, execute the corresponding keyboard action when it is
+  "Highlight the first occurence of [..] in STR.
+If FUNC-OR-SHORTCUT is non-nil and if it is a function, call it
+when STR is clicked (using RET or mouse-2); if FUNC-OR-SHORTCUT
+is a string, execute the corresponding keyboard action when it is
 clicked."
   (let ((newstr
 	  (replace-regexp-in-string
@@ -103,11 +107,13 @@ clicked."
 	(propertize  mu4e-mu-version 'face 'mu4e-view-header-key-face)
 
 	;; show some server properties; in this case; a big C when there's
-	;; crypto support
+	;; crypto support, a big G when there's Guile support
 	" "
-	(if (plist-get mu4e~server-props :crypto)
-	  (propertize "C" 'face 'mu4e-title-face)
-	  "")
+	(propertize
+	  (concat
+	    (when (plist-get mu4e~server-props :crypto) "C")
+	    (when (plist-get mu4e~server-props :guile)  "G"))
+	  'face 'mu4e-title-face)
 	"\n\n"
 	(propertize "  Basics\n\n" 'face 'mu4e-title-face)
 	(mu4e~main-action-str "\t* [j]ump to some maildir\n" 'mu4e-jump-to-maildir)
@@ -152,7 +158,7 @@ clicked."
   "Toggle sending mail mode, either queued or direct."
   (interactive)
   (unless (file-directory-p smtpmail-queue-dir)
-    (mu4e-error "`smtp-queue-dir' does not exist"))
+    (mu4e-error "`smtpmail-queue-dir' does not exist"))
   (setq smtpmail-queue-mail (not smtpmail-queue-mail))
   (message
     (concat "Outgoing mail will now be "
