@@ -31,33 +31,32 @@
 
 (defgroup mu4e nil
   "mu4e - mu for emacs"
-  :group 'local)
+  :group 'mail)
 
 (defcustom mu4e-mu-home nil
   "Location of the mu homedir, or nil for the default."
-  :type 'directory
   :group 'mu4e
   :type '(choice (const :tag "Default location" nil)
-                 (const :tag "Specify location" string))
+                 (directory :tag "Specify location"))
   :safe 'stringp)
 
 (defcustom mu4e-mu-binary (executable-find "mu")
-  "Name of the mu-binary to use; if it cannot be found in your
-PATH, you can specify the full path."
+  "Name of the mu-binary to use.
+If it cannot be found in your PATH, you can specify the full
+path."
   :type 'file
   :group 'mu4e
   :safe 'stringp)
 
 (defcustom mu4e-maildir (expand-file-name "~/Maildir")
-  "Your Maildir directory; by default, mu4e assumes
-~/Maildir."
+  "Your Maildir directory."
   :type 'directory
   :safe 'stringp
   :group 'mu4e)
 
 (defcustom mu4e-get-mail-command "true"
-  "Shell command to run to retrieve new mail; e.g. 'offlineimap' or
-'fetchmail'."
+  "Shell command to run to retrieve new mail.
+Common values are \"offlineimap\" and \"fetchmail\"."
   :type 'string
   :group 'mu4e
   :safe 'stringp)
@@ -73,78 +72,82 @@ mu4e."
   :safe 'integerp)
 
 (defcustom mu4e-attachment-dir (expand-file-name "~/")
-  "Default directory for saving attachments. This can be either a
-string, or a function that takes a filename FNAME and MIMETYPE as
-arguments, and returns the attachment dir. Note, either or both of
-the arguments may be `nil'."
+  "Default directory for saving attachments.
+This can be either a string, or a function that takes a filename
+FNAME and MIMETYPE as arguments, and returns the attachment
+dir. Note, either or both of the arguments may be `nil'."
   :type 'directory
   :group 'mu4e
   :safe 'stringp)
 
-(defcustom mu4e-user-mail-address-regexp "$^"
-  "Regular expression matching the user's mail address(es). This is
-used to distinguish ourselves from others, e.g. when replying and
-in :from-or-to headers. By default, match nothing."
-  :type 'regexp
-  :group 'mu4e
-  :safe 'stringp)
+(defcustom mu4e-user-mail-address-list `(,user-mail-address)
+  "List of e-mail addresses to consider 'my email addresses'.
+I.e. addresses whose presence in an email imply that it is a
+personal message. This is used when indexing messages."
+  :type '(repeat (string :tag "Address"))
+  :group 'mu4e)
+
+;; don't use the older vars anymore
+(make-obsolete-variable 'mu4e-user-mail-address-regexp
+  'mu4e-user-mail-address-list "0.9.9.x")
+
+(make-obsolete-variable 'mu4e-my-email-addresses
+  'mu4e-user-mail-address-list "0.9.9.x")
 
 (defcustom mu4e-use-fancy-chars nil
   "Whether to use fancy (non-ascii) characters."
   :type 'boolean
   :group 'mu4e)
 
-(defcustom mu4e-my-email-addresses `(,user-mail-address)
-  "List of e-mail addresses to consider 'my email addresses',
-ie. addresses whose presence in an email imply that it is a
-personal message. This is used when indexing messages."
-  :type '(repeat (string :tag "Address"))
-  :group 'mu4e)
-
-(defvar mu4e-date-format-long "%c"
+(defcustom mu4e-date-format-long "%c"
   "Date format to use in the message view, in the format of
-  `format-time-string'.")
-
-(defcustom mu4e-search-results-limit 500
-  "Maximum number of search results (or -1 for unlimited). Since
-limiting search results speeds up searches significantly, it's
-useful to limit this. Note, to ignore the limit, use a prefix
-argument (C-u) before invoking the search."
-  :type '(choice (const :tag "Unlimited" -1)
-                 (integer :tag "Limit"))
+  `format-time-string'."
+  :type 'string
   :group 'mu4e)
 
+
 (defvar mu4e-debug nil
   "When set to non-nil, log debug information to the *mu4e-log* buffer.")
 
-(defvar mu4e-bookmarks
+(defcustom mu4e-bookmarks
   '( ("flag:unread AND NOT flag:trashed" "Unread messages"      ?u)
      ("date:today..now"                  "Today's messages"     ?t)
      ("date:7d..now"                     "Last 7 days"          ?w)
      ("mime:image/*"                     "Messages with images" ?p))
-  "A list of pre-defined queries; these will show up in the main
-screen. Each of the list elements is a three-element list of the
-form (QUERY DESCRIPTION KEY), where QUERY is a string with a mu
-query, DESCRIPTION is a short description of the query (this will
-show up in the UI), and KEY is a shortcut key for the query.")
+  "A list of pre-defined queries.
+These will show up in the main screen. Each of the list elements
+is a three-element list of the form (QUERY DESCRIPTION KEY),
+where QUERY is a string with a mu query, DESCRIPTION is a short
+description of the query (this will show up in the UI), and KEY
+is a shortcut key for the query."
+  :type '(repeat (list (string :tag "Query")
+		   (string :tag "Description")
+		   character))
+  :group 'mu4e)
 
 (defcustom mu4e-split-view 'horizontal
-  "How to show messages / headers; a symbol which is either: * a
-symbol 'horizontal: split horizontally (headers on top) * a symbol
-'vertical: split vertically (headers on the left).  * anything
-else: don't split (show either headers or messages, not both) Also
-see `mu4e-headers-visible-lines' and
-`mu4e-headers-visible-columns'."
+  "How to show messages / headers.
+A symbol which is either:
+ * `horizontal':   split horizontally (headers on top)
+ * `vertical':     split vertically (headers on the left).
+ * anything else:  don't split (show either headers or messages,
+                  not both)
+Also see `mu4e-headers-visible-lines'
+and `mu4e-headers-visible-columns'."
   :type '(choice (const :tag "Split horizontally" horizontal)
                  (const :tag "Split vertically" vertical)
-                 (const :tag "Don't split" nil)))
+                 (const :tag "Don't split" nil))
+  :group 'mu4e-headers)
 
-(defcustom mu4e-show-images nil
+(defcustom mu4e-view-show-images nil
   "Whether to automatically display attached images in the message
 view buffer."
   :type 'boolean
   :group 'mu4e-view)
 
+(make-obsolete-variable 'mu4e-show-images
+  'mu4e-view-show-images "0.9.9.x")
+
 (defcustom mu4e-confirm-quit t
   "Whether to confirm to quit mu4e."
   :type 'boolean
@@ -161,7 +164,8 @@ view buffer."
   :group 'mu4e-crypto)
 
 (defcustom mu4e-decryption-policy t
-  "Policy for dealing with encrypted parts. The setting is a symbol:
+  "Policy for dealing with encrypted parts.
+The setting is a symbol:
  * t:     try to decrypt automatically
  * `ask': ask before decrypting anything
  * nil:   don't try to decrypt anything."
@@ -193,19 +197,38 @@ details (in particular, how to define your own e-mail addresses)."
   :group 'mu4e-compose)
 
 (defcustom mu4e-compose-complete-only-after "2010-01-01"
-  "Consider only contacts last seen after this date. Date must be a
-  string, in a format parseable by `org-parse-time-string'. This
-  excludes really old contacts. Set to nil to not have any
-  time-based restriction."
+  "Consider only contacts last seen after this date.
+Date must be a string, in a format parseable by
+`org-parse-time-string'. This excludes really old contacts.
+Set to nil to not have any time-based restriction."
   :type 'string
   :group 'mu4e-compose)
 
 (defcustom mu4e-compose-complete-ignore-address-regexp "no-?reply"
-  "Ignore any e-mail addresses for completion if they match this
-regexp."
+  "Ignore any e-mail addresses for completion if they match this regexp."
   :type 'string
   :group 'mu4e-compose)
 
+(defcustom mu4e-compose-reply-to-address nil
+  "The Reply-To address (if this, for some reason, is not equal to
+the From: address.)"
+  :type 'string
+  :group 'mu4e-compose)
+
+;; backward compatibility
+(make-obsolete-variable 'mu4e-reply-to-address 'mu4e-compose-reply-to-address
+  "v0.9.9")
+
+(defcustom mu4e-compose-keep-self-cc nil
+  "Non-nil means your e-mail address is kept on the CC list when
+replying to messages."
+  :type 'boolean
+  :group 'mu4e-compose)
+
+(defvar mu4e-compose-parent-message nil
+  "The parent message plist.
+This is the message being replied to, forwarded or edited; used
+in `mu4e-compose-pre-hook'. For new messages, it is nil.")
 
 ;; Folders
 (defgroup mu4e-folders nil
@@ -213,14 +236,15 @@ regexp."
   :group 'mu4e)
 
 (defcustom mu4e-drafts-folder "/drafts"
-  "Your folder for draft messages, relative to `mu4e-maildir',
+  "Your folder for draft messages, relative to `mu4e-maildir'.
 e.g. \"/drafts\". Instead of a string, may also be a function that
 takes a message (a msg plist, see `mu4e-message-get-field'), and
 returns a folder.  Note, the message parameter refers to the
 original message being replied to / being forwarded / re-edited and
 is nil otherwise. `mu4e-drafts-folder' is only evaluated once."
-  :type 'string
-  :safe 'stringp
+  :type '(choice
+	   (string :tag "Folder name")
+	   (function :tag "Function return folder name"))
   :group 'mu4e-folders)
 
 (defcustom mu4e-refile-folder "/archive"
@@ -229,8 +253,9 @@ e.g. \"/Archive\". Instead of a string, may also be a function that
 takes a message (a msg plist, see `mu4e-message-get-field'), and
 returns a folder. Note that the message parameter refers to the
 message-at-point."
-  :type 'string
-  :safe 'stringp
+  :type '(choice
+	   (string :tag "Folder name")
+	   (function :tag "Function return folder name"))
   :group 'mu4e-folders)
 
 (defcustom mu4e-sent-folder "/sent"
@@ -240,8 +265,9 @@ that takes a message (a msg plist, see `mu4e-message-get-field'),
 and returns a folder.  Note that the message parameter refers to
 the original message being replied to / being forwarded /
 re-edited, and is nil otherwise."
-  :type 'string
-  :safe 'stringp
+  :type '(choice
+	   (string :tag "Folder name")
+	   (function :tag "Function return folder name"))
   :group 'mu4e-folders)
 
 (defcustom mu4e-trash-folder "/trash"
@@ -254,23 +280,24 @@ parameter refers to the message-at-point. When using it when
 composing a message (see `mu4e-sent-messages-behavior'), this
 refers to the original message being replied to / being forwarded /
 re-edited, and is nil otherwise."
-  :type 'string
-  :safe 'stringp
+  :type '(choice
+	   (string :tag "Folder name")
+	   (function :tag "Function return folder name"))
   :group 'mu4e-folders)
 
 
 (defcustom mu4e-maildir-shortcuts nil
-  "A list of maildir shortcuts to enable quickly going to the
- particular for, or quickly moving messages towards them (i.e.,
- archiving or refiling). The list contains elements of the form
- (maildir . shortcut), where MAILDIR is a maildir (such as
-\"/archive/\"), and shortcut a single shortcut character. With
-this, in the header buffer and view buffer you can execute
-`mu4e-mark-for-move-quick' (or 'm', by default) or
-`mu4e-jump-to-maildir' (or 'j', by default), followed by the
-designated shortcut character for the maildir.")
-
-
+  "A list of maildir shortcuts.
+This enables quickly going to the particular for, or quickly
+moving messages towards them (i.e., archiving or refiling). The
+list contains elements of the form (maildir . shortcut), where
+MAILDIR is a maildir (such as \"/archive/\"), and shortcut a
+single shortcut character. With this, in the header buffer and
+view buffer you can execute `mu4e-mark-for-move-quick' (or 'm',
+by default) or `mu4e-jump-to-maildir' (or 'j', by default),
+followed by the designated shortcut character for the maildir."
+  :type '(repeat (cons (string :tag "Maildir") character))
+  :group 'mu4e-folders)
 
 ;; Faces
 (defgroup mu4e-faces nil
@@ -285,8 +312,8 @@ designated shortcut character for the maildir.")
 
 (defface mu4e-moved-face
   '((t :inherit font-lock-comment-face :slant italic))
-  "Face for a message header that has been moved to some
-folder (it's still visible in the search results, since we cannot
+  "Face for a message header that has been moved to some folder.
+\(It's still visible in the search results, since we cannot
 be sure it no longer matches)."
   :group 'mu4e-faces)
 
@@ -297,8 +324,8 @@ be sure it no longer matches)."
 
 (defface mu4e-draft-face
   '((t :inherit font-lock-string-face))
-  "Face for a draft message header (i.e., a message with the draft
-flag set)."
+  "Face for a draft message header
+I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-flagged-face
@@ -322,7 +349,7 @@ flag set)."
   :group 'mu4e-faces)
 
 (defface mu4e-header-highlight-face
-  '((t :inherit default :weight bold :underline t))
+  '((t :inherit region :weight bold :underline t))
   "Face for the header at point."
   :group 'mu4e-faces)
 
@@ -339,8 +366,7 @@ flag set)."
 
 (defface mu4e-view-header-value-face
   '((t :inherit font-lock-doc-face))
-  "Face for a header value (such as \"Re: Hello!\") in the message
-  view."
+  "Face for a header value (such as \"Re: Hello!\") in the message view."
   :group 'mu4e-faces)
 
 (defface mu4e-view-special-header-value-face
@@ -400,7 +426,7 @@ flag set)."
   :group 'mu4e-faces)
 
 (defface mu4e-cited-4-face
-  '((t :inherit font-lock-pseudo-keyword-face :bold nil :italic t))
+  '((t :inherit font-lock-keyword-face :bold nil :italic t))
   "Face for cited message parts (level 4)."
   :group 'mu4e-faces)
 
@@ -422,8 +448,7 @@ flag set)."
 
 (defface mu4e-system-face
   '((t :inherit font-lock-comment-face :slant italic))
-  "Face for system message (such as the footers for message
-headers)."
+  "Face for system message (such as the footers for message headers)."
   :group 'mu4e-faces)
 
 (defface mu4e-ok-face
@@ -460,6 +485,11 @@ headers)."
 	 :shortname "Date"
 	 :help "Date/time when the message was written"
 	 :sortable t))
+     (:human-date .
+       ( :name "Date"
+	 :shortname "Date"
+	 :help "Date/time when the message was written."
+	 :sortable :date))
      (:flags .
        ( :name "Flags"
 	 :shortname "Flgs"
@@ -473,14 +503,18 @@ headers)."
      (:from-or-to .
        ( :name "From/To"
 	 :shortname "From/To"
-	 :help "Sender of the message if it's not me; otherwise
-	 the recipient"
+	 :help "Sender of the message if it's not me; otherwise the recipient"
 	 :sortable nil))
      (:maildir .
        ( :name "Maildir"
 	 :shortname "Maildir"
 	 :help "Maildir for this message"
 	 :sortable t))
+     (:mailing-list .
+       ( :name "List"
+	 :shortname "List"
+	 :help "Mailing list for this message"
+	 :sortable nil))
      (:path .
        ( :name "Path"
 	 :shortname "Path"
@@ -501,24 +535,36 @@ headers)."
 	 :shortname "Subject"
 	 :help "Subject of the message"
 	 :sortable t))
+     (:tags .
+       ( :name "Tags"
+	 :shortname "Tags"
+	 :help "Tags for the message"
+	 :sortable nil))
      (:to .
        ( :name "To"
 	 :shortname "T"
 	 :help "Recipient of the message"
 	 :sortable t)))
-  "An alist of all possible header fields and information about
-them.; this is used in the UI (the column headers in the header
-list, and the fields the message view). Most fields should be
-self-explanatory. A special one is `:from-or-to', which is equal to
-`:from' unless `:from' matches `mu4e-user-mail-address-regexp', in
-which case it will be equal to `:to'.")
+  "An alist of all possible header fields and information about them.
+This is used in the user-interface (the column headers in the header list, and
+the fields the message view).
 
+Most fields should be self-explanatory. A special one is
+`:from-or-to', which is equal to `:from' unless `:from' matches
+`mu4e-user-mail-address-regexp', in which case it will be equal to
+`:to'.
+
+Furthermore, the property `:sortable' determines whether we can
+sort by this field.  This can be either a boolean (nil or t), or a
+symbol for /another/ field. For example, the `:human-date' field
+uses `:date' for that.
+")
 
 (defvar mu4e-custom-header-info nil
-  "A list like `mu4e-custom-header-info', but for
-custom (user-specified) headers. Each of the list items is a
-property list with :name (the full-name, as displayed in the
-message view), :shortname (the name as displayed in the headers
+  "A list like `mu4e-custom-header-info', but for custom headers.
+I.e. user-specified headers. Each of the list items is a property
+list with :name (the full-name, as displayed in the message
+view), :shortname (the name as displayed in the headers
 view), :help (some help information, which shows up in the
 tooltip). Furthermore, there are two special fields:
 :headers-func and :message-func, and the values should be functions
@@ -550,17 +596,18 @@ Note, :sortable does not work for custom header fields.")
 (defvar mu4e~view-msg nil "The message being viewed in view mode.")
 
 (defvar mu4e~contacts-for-completion nil
-  "List of contacts (ie. 'name <e-mail>'),
-used by the completion functions in mu4e-compose, and filled when
-mu4e starts.")
+  "List of contacts (ie. 'name <e-mail>').
+This is used by the completion functions in mu4e-compose, and
+filled when mu4e starts.")
 
 (defvar mu4e~server-props nil
-  "Properties we receive from the mu4e server process (in the
-  'pong-handler').")
+  "Properties we receive from the mu4e server process.
+\(in the 'pong-handler').")
 
 (defvar mu4e~headers-last-query nil
   "The present (most recent) query.")
 
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; our handlers funcs
